#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "feedforwardnetwork.h"
#include "csvparser.h"

#define NUM_INPUTS 6318

void print_msg(const char *type, const char *msg) {
	if (strcmp(type, "E") == 0 || strcmp(type, "W") == 0) {
		fprintf(stderr, "evaluate -%s- %s\n", type, msg);
	} else {
		printf("evaluate -%s- %s\n", type, msg);
	}
}

void exit_error(const char *msg) {
	print_msg("E", "Fatal Error:");
	print_msg("E", msg);
	print_msg("W", "Terminating...");
	exit(1);
}

double **getInputsFromCsv(const char *filepath, int *num_samples_ref) {
	double **samples = NULL;
	*num_samples_ref = 0;
	CsvParser *csvparser = CsvParser_new(filepath, ",", 0);
	CsvRow *row;
	while ((row = CsvParser_getRow(csvparser)) ) {

		int num_fields = CsvParser_getNumFields(row);
		if (num_fields != NUM_INPUTS) {
			fprintf(stderr, "Error in CSV file: %s in line %d : Number of fields is %d instead of %d\n", filepath, *num_samples_ref, num_fields, NUM_INPUTS);
			exit(1);	
		}		
		//printf("NEW LINE:\n");
		samples = realloc(samples, ((*num_samples_ref) + 1) * sizeof(double*));
		samples[*num_samples_ref] = malloc(NUM_INPUTS * sizeof(double));		
		char **rowFields = CsvParser_getFields(row);
		int i;

		for (i = 0 ; i < num_fields ; i++) {
			samples[*num_samples_ref][i] = atof(rowFields[i]);
			//printf("FIELD: %s\n", rowFields[i]);
		}
		CsvParser_destroy_row(row);
		(*num_samples_ref)++;
	}
	CsvParser_destroy(csvparser);

	return samples;

}

int main(int argc, char **argv) {
	if (argc != 3) {
		print_msg("E", "Usage error: Please supply input file");
		exit(2);
	}
	const char *network_file = argv[1];
	const char *samples_csv_file = argv[2];

	FeedForwardNetwork *ffn = FeedForwardNetwork_new_from_file(network_file);
	if (ffn == NULL) {
		exit_error("Error initializing network...");
	}
	int num_samples = 0;
	double **samples = getInputsFromCsv(samples_csv_file, &num_samples);
	int i;
	for (i = 0 ; i < num_samples ; i++) {
		double *results = FeedForwardNetwork_activate(ffn, samples[i]);
		const char *ans = (results[0] >= 0.5) ? "1" : "0";
		puts(ans);
	}
	return 0;
}










